package istia.st.m2vc.core;

import java.util.Map;

/**
 * @author serge.tahe@univ-angers.fr, juin 2005
 *
 */
public class BaseControleur
    implements IControleur {

  // champs privs
  private Map actions; // les actions  contrler
  private String firstActionName; // le nom de la 1re action
  private String lastActionName; // le nom de la dernire action
  private Barriere synchro; // l'outil de synchronisation contrleur - vues

  /**
   * @return Returns the actions.
   */
  public Map getActions() {
    return actions;
  }

  /**
   * @param actions
   *          The actions to set.
   */
  public void setActions(Map actions) {
    this.actions = actions;
  }

  /**
   * @return Returns the firstActionName.
   */
  public String getFirstActionName() {
    return firstActionName;
  }

  /**
   * @param firstActionName
   *          The firstActionName to set.
   */
  public void setFirstActionName(String firstActionName) {
    this.firstActionName = firstActionName;
  }

  /**
   * @return Returns the lastActionName.
   */
  public String getLastActionName() {
    return lastActionName;
  }

  /**
   * @param lastActionName
   *          The lastActionName to set.
   */
  public void setLastActionName(String lastActionName) {
    this.lastActionName = lastActionName;
  }

  /**
   * @return Returns the synchro.
   */
  public Barriere getSynchro() {
    return synchro;
  }

  /**
   * @param synchro
   *          The synchro to set.
   */
  public void setSynchro(Barriere synchro) {
    this.synchro = synchro;
  }

  // le moteur d'excution des actions
  public void run() {

    // variables locales
    InfosAction configAction = null;
    String actionName = null;
    String tat = null;
    IVue vue = null;
    IVue vuePrcdente = null;

    // boucle d'excution des actions
    actionName = firstActionName;
    while (!actionName.equals(lastActionName)) {
      // on rcupre la config de l'action
      configAction = (InfosAction) actions.get(actionName);
      if (configAction == null) {
        // erreur de config - on s'arrte
        throw new RuntimeException("L'action [" + actionName
                                   + "] n'a pas t configure correctement");
      }
      // excution de l'action s'il y en a une
      if (configAction.getAction() != null) {
        // excution de l'action
        tat = configAction.getAction().execute();
        // on rcupre la vue associe  l'tat
        vue = (IVue) configAction.getEtats().get(tat);
        // si vue == null, erreur de config
        if (vue == null) {
          throw new RuntimeException("L'tat [" + tat + "] de l'action ["
                                     + actionName +
                                     "] n'a pas t configur correctement");
        }
      }
      else {
        // pas d'action - directement la vue
        tat = "";
        vue = configAction.getVue();
      }
      // on cache la vue prcdente si elle est diffrente de celle qui va tre
      // affiche
      if (vue != vuePrcdente && vuePrcdente != null) {
        vuePrcdente.cache();
      }
      // on initialise la vue  afficher
      initVue(actionName, tat, vue.getNom());
      // on affiche la vue en se synchronisant avec elle
      synchro.reset();
      vue.affiche();
      synchro.waitOne();
      // action suivante
      actionName = vue.getAction();
      vuePrcdente = vue;
    }
    // on cache la dernire vue affiche
    if (vue != null) {
      vue.cache();
    }
    // c'est fini
  }

  public void initVue(String action, String tat, String vue) {
    // action dlgue aux classes drives
  }

}
